package geniusweb.domaineditor.model;

import geniusweb.issuevalue.DiscreteValueSet;
import geniusweb.issuevalue.NumberValueSet;
import geniusweb.issuevalue.ValueSet;
import tudelft.utilities.logging.Reporter;

/**
 * Types of {@link ValueSetModel} and Model factory
 *
 */
public enum ValueSetModelType {
	DISCRETE(DiscreteValueSetModel.class), NUMBER(NumberValueSetModel.class);

	private Class<? extends ValueSetModel> modelclass;

	/**
	 * 
	 * @param clas the model class matching with this type.
	 */
	ValueSetModelType(Class<? extends ValueSetModel> clas) {
		this.modelclass = clas;
	}

	/**
	 * 
	 * @param valueset the {@link ValueSet} to get type of
	 * @return the type
	 */
	public static ValueSetModelType fromModel(ValueSet valueset) {
		if (valueset instanceof DiscreteValueSet)
			return DISCRETE;
		if (valueset instanceof NumberValueSet)
			return NUMBER;
		throw new IllegalArgumentException(
				"valueset of Unknown class " + valueset);
	}

	/**
	 * Find the Type for the given model.
	 * 
	 * @param model {@link ValueSetModel}
	 * @return {@link ValueSetModelType}
	 */
	public static ValueSetModelType fromModel(ValueSetModel model) {
		for (ValueSetModelType type : ValueSetModelType.values()) {
			if (model.getClass().equals(type.modelclass))
				return type;
		}
		throw new IllegalArgumentException("Model of Unknown class " + model);
	}

	public static ValueSetModel createModel(ValueSetModelType type,
			Reporter log) {
		switch (type) {
		case DISCRETE:
			return new DiscreteValueSetModel(log);
		case NUMBER:
			return new NumberValueSetModel(log);
		}
		return null; // unreachable code
	}

	/**
	 * 
	 * @return the other type
	 */
	public ValueSetModelType otherType() {
		return this == DISCRETE ? NUMBER : DISCRETE;
	}
}
